#!/pro/bin/perl -w

package PROCURA::Cache::BEL;

use strict;
use Exporter;
use Carp;
use vars   qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

$VERSION = "1.01";
@ISA     = ("Exporter", scalar caller);
@EXPORT  = qw(s_land s_plaats s_straat
	      $diac_ok
	      );

@EXPORT_OK   = (@EXPORT, qw(maxcache cachestats stopcache));
%EXPORT_TAGS = (
    util	=> [ qw(maxcache cachestats stopcache) ],
    "03"	=> [ qw(s_plaats s_land) ],
    "11"	=> [ qw(s_straat) ],
    );

use PROCURA::Diac;
use PROCURA::DBD;

my %cache;

INIT {

    DBDlogon ();	# Presume client using DBD will logoff

    ### ###########################################################################

    %cache = (	# [ Requests, Misses, Cache Size, $sth ]
	land	=> [ 0, 0,   75, prepar ("select land,   k_land",
					 "from   land",
					 "where  c_land     = ?")	],
	plaats	=> [ 0, 0,  150, prepar ("select plaats, k_plaats",
					 "from   plaats",
					 "where  c_plaats   = ?")	],
	straat	=> [ 0, 0,  150, prepar ("select straat, k_straat",
					 "from   straat",
					 "where  c_straat   = ?")	],
	diac	=> [ 0, 0,    0, prepar ("select diac",
					 "from   diac",
					 "where  c_diac     = ?")	],
	);
    }

sub stopcache ()
{
    foreach my $r (keys %cache) {
	my $sth = $cache{$r}[3] or next;
	$sth->finish ();
	$cache{$r}[3] = undef;
	}
    } # stopcache

END { stopcache (); }

sub maxcache ($;$)
{
    my ($t, $mc) = @_;
    exists $cache{$t} or carp "Table $t is not cached";
    defined $mc or return $cache{$t}[2];
    $cache{$t}[2] = $mc + 0;
    } # maxcache

sub cachestats ()
{
    print STDERR "\n";
    print STDERR "Table      Cache   Asked  Misses    Hits  HitRate\n";
    print STDERR "========== ===== ======= ======= ======= ========\n";
    foreach my $tab (sort { $cache{$b}[0] <=> $cache{$a}[0] } keys %cache) {
	$tab eq "diac" and next;
	my ($ask, $miss, $csz) = @{$cache{$tab}};
	my $rate = $ask ? 100 * ($ask - $miss) / $ask : 100.0;
	printf STDERR "%-10s %5d %7d %7d %7d %6.2f %%\n",
	    $tab, $csz, $ask, $miss, $ask - $miss, $rate;
	}
    } # cachestats

use vars qw($diac_ok);
$diac_ok = 1;
sub diac ($$$$)
{
    my ($f, $c, $k, $d) = @_;

    $diac_ok or return $f;
    my $sth = $cache{diac}[3];
    $sth->execute ($k);
    my ($diac) = $sth->fetchrow_array ();
    $diac and return diacgba ($f, $diac);
    warn ucfirst ($d), " $c DIAC empty!";
    $f;
    } # diac

{   my %land;
    my @land;
    sub s_land ($)
    {
	my $c_land = shift;

	$cache{land}[0]++;

	defined $c_land or  return "";
	$c_land ==   -1 and return "";
	$c_land == 6030 and return "Nederland";
	exists $land{$c_land} and return $land{$c_land};

	$cache{land}[1]++;

	@land == $cache{land}[2] and delete $land{shift @land};

	my $sth = $cache{land}[3];
	$sth->execute ($c_land);
	my ($land, $c_diac) = $sth->fetchrow_array ();
	$c_diac >= 0 and $land = diac ($land, $c_land, $c_diac, "land");
	push @land, $c_land;
	$land{$c_land} = $land;	# Also return value
	} # s_land
    }

{   my %plaats;
    my @plaats;
    sub s_plaats ($)
    {
	my $c_plaats = shift;

	$cache{plaats}[0]++;

	defined $c_plaats or  return "";
	$c_plaats == -1   and return "";
	exists $plaats{$c_plaats} and return $plaats{$c_plaats};

	$cache{plaats}[1]++;

	@plaats == $cache{plaats}[2] and delete $plaats{shift @plaats};

	my $sth = $cache{plaats}[3];
	$sth->execute ($c_plaats);
	my ($plaats, $c_diac) = $sth->fetchrow_array ();
	$c_diac >= 0 and $plaats = diac ($plaats, $c_plaats, $c_diac, "plaats");
	push @plaats, $c_plaats;
	$plaats{$c_plaats} = $plaats;	# Also return value
	} # s_plaats
    }

{   my %straat;
    my @straat;
    sub s_straat ($)
    {
	my $c_straat = shift;

	$cache{straat}[0]++;

	defined $c_straat or  return "";
	$c_straat == -1   and return "";
	exists $straat{$c_straat} and return $straat{$c_straat};

	$cache{straat}[1]++;

	@straat == $cache{straat}[2] and delete $straat{shift @straat};

	my $sth = $cache{straat}[3];
	$sth->execute ($c_straat);
	my ($straat, $c_diac) = $sth->fetchrow_array ();
	$c_diac >= 0 and $straat = diac ($straat, $c_straat, $c_diac, "straat");
	push @straat, $c_straat;
	$straat{$c_straat} = $straat;	# Also return value
	} # s_straat
    }

{   my %voorv;
    my @voorv;
    sub s_voorv ($)
    {
	my $c_voorv = shift;

	$cache{voorv}[0]++;

	defined $c_voorv or  return "";
	$c_voorv == -1   and return "";
	exists $voorv{$c_voorv} and return $voorv{$c_voorv};

	$cache{voorv}[1]++;

	@voorv == $cache{voorv}[2] and delete $voorv{shift @voorv};

	my $sth = $cache{voorv}[3];
	$sth->execute ($c_voorv);
	my ($voorv) = $sth->fetchrow_array ();
	push @voorv, $c_voorv;
	$voorv{$c_voorv} = $voorv;	# Also return value
	} # s_voorv
    }

1;
