#!/pro/bin/perl -w

package PROCURA::Cache::BEV;

use strict;
use Exporter;
use Carp;
use vars   qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

$VERSION = "1.03";
@ISA     = ("Exporter", scalar caller);
@EXPORT  = qw(s_land s_plaats  s_straat  s_gem_deel s_locatie s_stem
	      s_naam s_voorv   s_voorn   s_tp       s_natio   s_vk_vl
	      s_doc  s_reisdoc s_afnemer s_b_adres
	      $diac_ok
	      );

@EXPORT_OK   = (@EXPORT, qw(maxcache cachestats stopcache));
%EXPORT_TAGS = (
    util	=> [ qw(maxcache cachestats stopcache) ],
    "02"	=> [ qw(s_naam s_voorv s_voorn s_tp) ],
    "03"	=> [ qw(s_plaats s_land) ],
    "05"	=> [ qw(s_natio s_vk_vl) ],
    "10"	=> [ qw(s_gem_deel) ],
    "11"	=> [ qw(s_straat) ],
    "12"	=> [ qw(s_locatie) ],
    "14"	=> [ qw(s_afnemer) ],
    "35"	=> [ qw(s_reisdoc) ],
    "82"	=> [ qw(s_doc) ],
    );

use PROCURA::Diac;
use PROCURA::DBD;

DBDlogon ();	# Presume client using DBD will logoff

### ###########################################################################

my %cache = (	# [ Requests, Misses, Cache Size, $sth ]
    natio	=> [ 0, 0,   75, prepar ("select natio, diac",
					 "from   natio",
					 "where  c_natio    = ?")	],
    vk_vl	=> [ 0, 0,   75, prepar ("select vk_vl, diac",
					 "from   vk_vl",
					 "where  c_vk_vl    = ?")	],
    land	=> [ 0, 0,   75, prepar ("select land, diac",
					 "from   land",
					 "where  c_land     = ?")	],
    plaats	=> [ 0, 0,  150, prepar ("select plaats, diac",
					 "from   plaats",
					 "where  c_plaats   = ?")	],
    gem_deel	=> [ 0, 0,   10, prepar ("select gem_deel, diac",
					 "from   gem_deel",
					 "where  c_gem_deel = ?")	],
    straat	=> [ 0, 0,  150, prepar ("select straat,     diac,",
					 "       straat_l,   diac_l,",
					 "       straat_nen, diac_nen",
					 "from   straat",
					 "where  c_straat   = ?")	],
    locatie	=> [ 0, 0,   30, prepar ("select locatie, diac",
					 "from   locatie",
					 "where  c_locatie  = ?")	],
    stem	=> [ 0, 0,   20, prepar ("select stem, diac",
					 "from   stem",
					 "where  c_stem     = ?")	],
    naam	=> [ 0, 0,  500, prepar ("select naam, diac",
					 "from   naam",
					 "where  c_naam     = ?")	],
    voorv	=> [ 0, 0,   50, prepar ("select voorv",
					 "from   voorv",
					 "where  c_voorv    = ?")	],
    tp		=> [ 0, 0,   10, prepar ("select tp",
					 "from   tp",
					 "where  c_tp       = ?")	],
    voorn	=> [ 0, 0,  500, prepar ("select voorn, diac",
					 "from   voorn",
					 "where  c_voorn    = ?")	],
    doc		=> [ 0, 0,   50, prepar ("select doc, diac",
					 "from   doc",
					 "where  c_doc      = ?")	],
    reisdoc	=> [ 0, 0,   30, prepar ("select reisdoc, diac",
					 "from   reisdoc",
					 "where  c_reisdoc  = ?")	],
    afnemer	=> [ 0, 0,   20, prepar ("select naam, beperk, d_in",
					 "from   afnemer",
					 "where  afn    = ?",
					 "  and  d_in  <= ?",
					 "order  by d_in desc")		],
    diac	=> [ 0, 0,    0, prepar ("select diac",
					 "from   diac",
					 "where  veld       = ?",
					 "   and c_veld     = ?")	],
    b_adres     => [ 0, 0,   30, prepar ("select adr, diac",
                                         "from   b_adres",
                                         "where  c_b_adres  = ?")       ],
    );

sub stopcache ()
{
    foreach my $r (keys %cache) {
	my $sth = $cache{$r}[3] or next;
	$sth->finish;
	$cache{$r}[3] = undef;
	}
    } # stopcache

END { stopcache (); }

sub maxcache ($;$)
{
    my ($t, $mc) = @_;
    exists $cache{$t} or carp "Table $t is not cached";
    defined $mc or return $cache{$t}[2];
    $cache{$t}[2] = $mc + 0;
    } # maxcache

sub cachestats ()
{
    print STDERR "\n";
    print STDERR "Table      Cache   Asked  Misses    Hits  HitRate\n";
    print STDERR "========== ===== ======= ======= ======= ========\n";
    foreach my $tab (sort { $cache{$b}[0] <=> $cache{$a}[0] } keys %cache) {
	$tab eq "diac" and next;
	my ($ask, $miss, $csz) = @{$cache{$tab}};
	my $rate = $ask ? 100 * ($ask - $miss) / $ask : 100.0;
	printf STDERR "%-10s %5d %7d %7d %7d %6.2f %%\n",
	    $tab, $csz, $ask, $miss, $ask - $miss, $rate;
	}
    } # cachestats

use vars qw($diac_ok);
$diac_ok = 1;
sub diac ($$$$)
{
    my ($f, $c, $v, $d) = @_;

    $diac_ok or return $f;
    my $sth = $cache{diac}[3];
    $sth->execute ($v, $c);
    my ($diac) = $sth->fetchrow_array;
    $diac and return diacgba ($f, $diac);
    warn ucfirst ($d), " $c DIAC empty!";
    $f;
    } # diac

{   my %land;
    my @land;
    sub s_land ($)
    {
	my $c_land = shift;

	$cache{land}[0]++;

	defined $c_land or  return "";
	$c_land ==   -1 and return "";
	$c_land == 6030 and return "Nederland";
	exists $land{$c_land} and return $land{$c_land};

	$cache{land}[1]++;

	@land == $cache{land}[2] and delete $land{shift @land};

	my $sth = $cache{land}[3];
	$sth->execute ($c_land);
	my ($land, $diac) = $sth->fetchrow_array;
	s/\s+$// for $land, $diac;	# Oracle
	$diac and $land = diac ($land, $c_land, "v_003402", "land");
	push @land, $c_land;
	$land{$c_land} = $land;	# Also return value
	} # s_land
    }

{   my %natio;
    my @natio;
    sub s_natio ($)
    {
	my $c_natio = shift;

	$cache{natio}[0]++;

	defined $c_natio or  return "";
	$c_natio ==   -1 and return "";
	$c_natio ==    0 and return "Onbekend";
	$c_natio ==    1 and return "Nederlandse";
	exists $natio{$c_natio} and return $natio{$c_natio};

	$cache{natio}[1]++;

	@natio == $cache{natio}[2] and delete $natio{shift @natio};

	my $sth = $cache{natio}[3];
	$sth->execute ($c_natio);
	my ($natio, $diac) = $sth->fetchrow_array;
	s/\s+$// for $natio, $diac;	# Oracle
	$diac and $natio = diac ($natio, $c_natio, "v_003202", "natio");
	push @natio, $c_natio;
	$natio{$c_natio} = $natio;	# Also return value
	} # s_natio
    }

{   my %vk_vl;
    my @vk_vl;
    sub s_vk_vl ($)
    {
	my $c_vk_vl = shift;

	$cache{vk_vl}[0]++;

	defined $c_vk_vl or  return "";
	$c_vk_vl ==   -1 and return "";
	$c_vk_vl ==    0 and return "Onbekend";
	exists $vk_vl{$c_vk_vl} and return $vk_vl{$c_vk_vl};

	$cache{vk_vl}[1]++;

	@vk_vl == $cache{vk_vl}[2] and delete $vk_vl{shift @vk_vl};

	my $sth = $cache{vk_vl}[3];
	$sth->execute ($c_vk_vl);
	my ($vk_vl, $diac) = $sth->fetchrow_array;
	s/\s+$// for $vk_vl, $diac;	# Oracle
	$diac and $vk_vl = diac ($vk_vl, $c_vk_vl, "v_003701", "vk_vl");
	push @vk_vl, $c_vk_vl;
	$vk_vl{$c_vk_vl} = $vk_vl;	# Also return value
	} # s_vk_vl
    }

{   my %plaats;
    my @plaats;
    sub s_plaats ($)
    {
	my $c_plaats = shift;

	$cache{plaats}[0]++;

	defined $c_plaats or  return "";
	$c_plaats == -1   and return "";
	exists $plaats{$c_plaats} and return $plaats{$c_plaats};

	$cache{plaats}[1]++;

	@plaats == $cache{plaats}[2] and delete $plaats{shift @plaats};

	my $sth = $cache{plaats}[3];
	$sth->execute ($c_plaats);
	my ($plaats, $diac) = $sth->fetchrow_array;
	s/\s+$// for $plaats, $diac;	# Oracle
	$diac and $plaats = diac ($plaats, $c_plaats, "v_003302", "plaats");
	push @plaats, $c_plaats;
	$plaats{$c_plaats} = $plaats;	# Also return value
	} # s_plaats
    }

{   my %gem_deel;
    my @gem_deel;
    sub s_gem_deel ($)
    {
	my $c_gem_deel = shift;

	$cache{gem_deel}[0]++;

	defined $c_gem_deel or  return "";
	$c_gem_deel == -1   and return "";
	exists $gem_deel{$c_gem_deel} and return $gem_deel{$c_gem_deel};

	$cache{gem_deel}[1]++;

	@gem_deel == $cache{gem_deel}[2] and delete $gem_deel{shift @gem_deel};

	my $sth = $cache{gem_deel}[3];
	$sth->execute ($c_gem_deel);
	my ($gem_deel, $diac) = $sth->fetchrow_array;
	s/\s+$// for $gem_deel, $diac;	# Oracle
	$diac and $gem_deel = diac ($gem_deel, $c_gem_deel, "v_00bd02", "gem_deel");
	push @gem_deel, $c_gem_deel;
	$gem_deel{$c_gem_deel} = $gem_deel;	# Also return value
	} # s_gem_deel
    }

{   my %straat;
    my @straat;
    sub s_straat ($)
    {
	my $c_straat = shift;
	my @s = ("", "", "");	# straat, straat_l, straat_nen

	$cache{straat}[0]++;

	if (defined $c_straat && $c_straat != -1) {
	    if (exists $straat{$c_straat}) {
		@s = @{$straat{$c_straat}};
		}
	    else {
		$cache{straat}[1]++;

		@straat == $cache{straat}[2] and delete $straat{shift @straat};

		my $sth = $cache{straat}[3];
		$sth->execute ($c_straat);
		my ($straat, $diac, $straat_l, $diac_l,
		    $straat_nen, $diac_nen) = $sth->fetchrow_array;
		s/\s+$// for $straat, $diac, $straat_l, $diac_l,
			     $straat_nen, $diac_nen;
		$diac     and $straat     = diac ($straat,     $c_straat, "v_00bc02", "straat");
		$diac_l   and $straat_l   = diac ($straat_l,   $c_straat, "v_00bc05", "straat");
		$diac_nen and $straat_nen = diac ($straat_nen, $c_straat, "v_00bc07", "straat");
		push @straat, $c_straat;
		@s = ($straat, $straat_l, $straat_nen);
		$straat{$c_straat} = [ @s ];
		}
	    }
	wantarray ? @s : $s[0];
	} # s_straat
    }

{   my %locatie;
    my @locatie;
    sub s_locatie ($)
    {
	my $c_locatie = shift;

	$cache{locatie}[0]++;

	defined $c_locatie or  return "";
	$c_locatie == -1   and return "";
	exists $locatie{$c_locatie} and return $locatie{$c_locatie};

	$cache{locatie}[1]++;

	@locatie == $cache{locatie}[2] and delete $locatie{shift @locatie};

	my $sth = $cache{locatie}[3];
	$sth->execute ($c_locatie);
	my ($locatie, $diac) = $sth->fetchrow_array;
	$diac and $locatie = diac ($locatie, $c_locatie, "v_00be01", "locatie");
	push @locatie, $c_locatie;
	$locatie{$c_locatie} = $locatie;	# Also return value
	} # s_locatie
    }

{   my %stem;
    my @stem;
    sub s_stem ($)
    {
	my $c_stem = shift;

	$cache{stem}[0]++;
	
	defined $c_stem or  return "";
	$c_stem == -1   and return "";
	exists $stem{$c_stem} and return $stem{$c_stem};

	$cache{stem}[1]++;

	@stem == $cache{stem}[2] and delete $stem{shift @stem};

	my $sth = $cache{stem}[3];
	$sth->execute ($c_stem);
	my ($stem, $diac) = $sth->fetchrow_array;
	$diac and $stem = diac ($stem, $c_stem, "v_00ar01", "stem");
	push @stem, $c_stem;
	$stem{$c_stem} = $stem;	# Also return value
	} # s_stem
    }

{   my %naam;
    my @naam;
    sub s_naam ($)
    {
	my $c_naam = shift;

	$cache{naam}[0]++;

	defined $c_naam or  return "";
	$c_naam == -1   and return "";
	exists $naam{$c_naam} and return $naam{$c_naam};

	$cache{naam}[1]++;

	@naam == $cache{naam}[2] and delete $naam{shift @naam};

	my $sth = $cache{naam}[3];
	$sth->execute ($c_naam);
	my ($naam, $diac) = $sth->fetchrow_array;
	$diac and $naam = diac ($naam, $c_naam, "v_00ai01", "naam");
	push @naam, $c_naam;
	$naam{$c_naam} = $naam;	# Also return value
	} # s_naam
    }

{   my %tp;
    my @tp;
    sub s_tp ($)
    {
	my $c_tp = shift;

	$cache{tp}[0]++;

	defined $c_tp or  return "";
	$c_tp == -1   and return "";
	exists $tp{$c_tp} and return $tp{$c_tp};

	$cache{tp}[1]++;

	@tp == $cache{tp}[2] and delete $tp{shift @tp};

	my $sth = $cache{tp}[3];
	$sth->execute ($c_tp);
	my ($tp) = $sth->fetchrow_array;
	push @tp, $c_tp;
	$tp{$c_tp} = $tp;	# Also return value
	} # s_tp
    }

{   my %voorv;
    my @voorv;
    sub s_voorv ($)
    {
	my $c_voorv = shift;

	$cache{voorv}[0]++;

	defined $c_voorv or  return "";
	$c_voorv == -1   and return "";
	exists $voorv{$c_voorv} and return $voorv{$c_voorv};

	$cache{voorv}[1]++;

	@voorv == $cache{voorv}[2] and delete $voorv{shift @voorv};

	my $sth = $cache{voorv}[3];
	$sth->execute ($c_voorv);
	my ($voorv) = $sth->fetchrow_array;
	push @voorv, $c_voorv;
	$voorv{$c_voorv} = $voorv;	# Also return value
	} # s_voorv
    }

{   my %voorn;
    my @voorn;
    sub s_voorn ($)
    {
	my $c_voorn = shift;

	$cache{voorn}[0]++;
	
	defined $c_voorn or  return "";
	$c_voorn == -1   and return "";
	exists $voorn{$c_voorn} and return $voorn{$c_voorn};

	$cache{voorn}[1]++;

	@voorn == $cache{voorn}[2] and delete $voorn{shift @voorn};

	my $sth = $cache{voorn}[3];
	$sth->execute ($c_voorn);
	my ($voorn, $diac) = $sth->fetchrow_array;
	$diac and $voorn = diac ($voorn, $c_voorn, "v_00ah01", "voorn");
	push @voorn, $c_voorn;
	$voorn{$c_voorn} = $voorn;	# Also return value
	} # s_voorn
    }

{   my %reisdoc;
    my @reisdoc;
    sub s_reisdoc ($)
    {
	my $c_reisdoc = shift;

	$cache{reisdoc}[0]++;

	defined $c_reisdoc or  return "";
	$c_reisdoc == -1   and return "";
	exists $reisdoc{$c_reisdoc} and return $reisdoc{$c_reisdoc};

	$cache{reisdoc}[1]++;

	@reisdoc == $cache{reisdoc}[2] and delete $reisdoc{shift @reisdoc};

	my $sth = $cache{reisdoc}[3];
	$sth->execute ($c_reisdoc);
	my ($reisdoc, $diac) = $sth->fetchrow_array;
	$diac and $reisdoc = diac ($reisdoc, $c_reisdoc, "v_004802", "reisdoc");
	push @reisdoc, $c_reisdoc;
	$reisdoc{$c_reisdoc} = $reisdoc;	# Also return value
	} # s_reisdoc
    }

{   my %doc;
    my @doc;
    sub s_doc ($)
    {
	my $c_doc = shift;

	$cache{doc}[0]++;

	defined $c_doc or  return "";
	$c_doc == -1 and return "";
	$c_doc ==  0 and return ".";
	$c_doc ==  1 and return "PK";
	$c_doc ==  2 and return "PKA";
	exists $doc{$c_doc} and return $doc{$c_doc};

	$cache{doc}[1]++;

	@doc == $cache{doc}[2] and delete $doc{shift @doc};

	my $sth = $cache{doc}[3];
	$sth->execute ($c_doc);
	my ($doc, $diac) = $sth->fetchrow_array;
	$diac and $doc = diac ($doc, $c_doc, "v_00bk01", "doc");
	push @doc, $c_doc;
	$doc{$c_doc} = $doc;	# Also return value
	} # s_doc
    }

{   my %afnemer;
    my @afnemer;
    my @t = localtime time;
    my $d_sys = ((1900 + $t[5]) * 100 + $t[4] + 1) * 100 + $t[3];
    sub s_afnemer ($;$)
    {
	my ($c_afnemer, $d_geld) = (@_, 0);

	$cache{afnemer}[0]++;

	my @s = ("", -1);
	if (defined $c_afnemer && $c_afnemer != -1) {
	    if (exists $afnemer{$c_afnemer}) {
		@s = @{$afnemer{$c_afnemer}};
		}
	    else {
		$cache{afnemer}[1]++;

		@afnemer == $cache{afnemer}[2] and delete $afnemer{shift @afnemer};

		$d_geld ||= $d_sys;
		my $sth = $cache{afnemer}[3];
		$sth->execute ($c_afnemer, $d_geld);
		my ($afnemer, $beperk, $d_in) = $sth->fetchrow_array;
		push @afnemer, $c_afnemer;
		defined $afnemer and @s = ($afnemer, $beperk);
		$afnemer{$c_afnemer} = [ @s ];	# Also return value
		}
	    }
	wantarray ? @s : $s[0];
	} # s_afnemer
    }


{   my %b_adres;
    my @b_adres;
    sub s_b_adres ($)
    {
        my $c_b_adres = shift;

        $cache{b_adres}[0]++;

        defined $c_b_adres or  return "";
        $c_b_adres == -1   and return "";
        exists $b_adres{$c_b_adres} and return $b_adres{$c_b_adres};

        $cache{b_adres}[1]++;

        @b_adres == $cache{b_adres}[2] and delete $b_adres{shift @b_adres};

        my $sth = $cache{b_adres}[3];
        $sth->execute ($c_b_adres);
        my ($b_adres, $diac) = $sth->fetchrow_array;
        $diac and $b_adres = diac ($b_adres, $c_b_adres, "b_adres", "b_adres");
        push @b_adres, $c_b_adres;
        $b_adres{$c_b_adres} = $b_adres;        # Also return value
        } # s_b_adres
    }


1;
