#!/pro/bin/perl -w

package PROCURA::Cache::BIS;

use strict;
use Exporter;
use Carp;
use vars   qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

$VERSION = "1.01";
@ISA     = ("Exporter", scalar caller);
@EXPORT  = qw(s_land s_plaats s_straat s_gem_deel s_locatie 
	      s_naam s_voorv  s_voorn  s_tp       s_natio   
	      s_doc
	      $diac_ok
	      );

@EXPORT_OK   = (@EXPORT, qw(maxcache cachestats stopcache));
%EXPORT_TAGS = (
    util	=> [ qw(maxcache cachestats stopcache) ],
    "02"	=> [ qw(s_naam s_voorv s_voorn s_tp) ],
    "03"	=> [ qw(s_plaats s_land) ],
    "05"	=> [ qw(s_natio) ],
    "10"	=> [ qw(s_gem_deel) ],
    "11"	=> [ qw(s_straat) ],
    "12"	=> [ qw(s_locatie) ],
    "82"	=> [ qw(s_doc) ],
    );

use PROCURA::Diac;
use PROCURA::DBD;

DBDlogon ();	# Presume client using DBD will logoff

### ###########################################################################

my %cache = (	# [ Requests, Misses, Cache Size, $sth ]
    natio	=> [ 0, 0,   75, prepar ("select natio, diac",
					 "from   PROBEV.natio",
					 "where  c_natio    = ?")	],
    land	=> [ 0, 0,   75, prepar ("select land, diac",
					 "from   PROBEV.land",
					 "where  c_land     = ?")	],
    plaats	=> [ 0, 0,  150, prepar ("select plaats, diac",
					 "from   PROBEV.plaats",
					 "where  c_plaats   = ?")	],
    gem_deel	=> [ 0, 0,   10, prepar ("select gem_deel, diac",
					 "from   PROBEV.gem_deel",
					 "where  c_gem_deel = ?")	],
    straat	=> [ 0, 0,  150, prepar ("select straat,     diac,",
					 "       straat_l,   diac_l,",
					 "       straat_nen, diac_nen",
					 "from   PROBEV.straat",
					 "where  c_straat   = ?")	],
    locatie	=> [ 0, 0,   30, prepar ("select locatie, diac",
					 "from   PROBEV.locatie",
					 "where  c_locatie  = ?")	],
    naam	=> [ 0, 0,  500, prepar ("select naam, diac",
					 "from   PROBEV.naam",
					 "where  c_naam     = ?")	],
    voorv	=> [ 0, 0,   50, prepar ("select voorv",
					 "from   PROBEV.voorv",
					 "where  c_voorv    = ?")	],
    tp		=> [ 0, 0,   10, prepar ("select tp",
					 "from   PROBEV.tp",
					 "where  c_tp       = ?")	],
    voorn	=> [ 0, 0,  500, prepar ("select voorn, diac",
					 "from   PROBEV.voorn",
					 "where  c_voorn    = ?")	],
    doc		=> [ 0, 0,   50, prepar ("select doc, diac",
					 "from   PROBEV.doc",
					 "where  c_doc      = ?")	],
    diac	=> [ 0, 0,    0, prepar ("select diac",
					 "from   PROBEV.diac",
					 "where  veld       = ?",
					 "   and c_veld     = ?")	],
    );

sub stopcache ()
{
    foreach my $r (keys %cache) {
	my $sth = $cache{$r}[3] or next;
	$sth->finish;
	$cache{$r}[3] = undef;
	}
    } # stopcache

END { stopcache (); }

sub maxcache ($;$)
{
    my ($t, $mc) = @_;
    exists $cache{$t} or carp "Table $t is not cached";
    defined $mc or return $cache{$t}[2];
    $cache{$t}[2] = $mc + 0;
    } # maxcache

sub cachestats ()
{
    print STDERR "\n";
    print STDERR "Table      Cache   Asked  Misses    Hits  HitRate\n";
    print STDERR "========== ===== ======= ======= ======= ========\n";
    foreach my $tab (sort { $cache{$b}[0] <=> $cache{$a}[0] } keys %cache) {
	$tab eq "diac" and next;
	my ($ask, $miss, $csz) = @{$cache{$tab}};
	my $rate = $ask ? 100 * ($ask - $miss) / $ask : 100.0;
	printf STDERR "%-10s %5d %7d %7d %7d %6.2f %%\n",
	    $tab, $csz, $ask, $miss, $ask - $miss, $rate;
	}
    } # cachestats

use vars qw($diac_ok);
$diac_ok = 1;
sub diac ($$$$)
{
    my ($f, $c, $v, $d) = @_;

    $diac_ok or return $f;
    my $sth = $cache{diac}[3];
    $sth->execute ($v, $c);
    my ($diac) = $sth->fetchrow_array;
    $diac and return diacgba ($f, $diac);
    warn ucfirst ($d), " $c DIAC empty!";
    $f;
    } # diac

{   my %land;
    my @land;
    sub s_land ($)
    {
	my $c_land = shift;

	$cache{land}[0]++;

	defined $c_land or  return "";
	$c_land ==   -1 and return "";
	$c_land == 6030 and return "Nederland";
	exists $land{$c_land} and return $land{$c_land};

	$cache{land}[1]++;

	@land == $cache{land}[2] and delete $land{shift @land};

	my $sth = $cache{land}[3];
	$sth->execute ($c_land);
	my ($land, $diac) = $sth->fetchrow_array;
	$diac and $land = diac ($land, $c_land, "v_003402", "land");
	push @land, $c_land;
	$land{$c_land} = $land;	# Also return value
	} # s_land
    }

{   my %natio;
    my @natio;
    sub s_natio ($)
    {
	my $c_natio = shift;

	$cache{natio}[0]++;

	defined $c_natio or  return "";
	$c_natio ==   -1 and return "";
	$c_natio ==    0 and return "Onbekend";
	$c_natio ==    1 and return "Nederlandse";
	exists $natio{$c_natio} and return $natio{$c_natio};

	$cache{natio}[1]++;

	@natio == $cache{natio}[2] and delete $natio{shift @natio};

	my $sth = $cache{natio}[3];
	$sth->execute ($c_natio);
	my ($natio, $diac) = $sth->fetchrow_array;
	$diac and $natio = diac ($natio, $c_natio, "v_003202", "natio");
	push @natio, $c_natio;
	$natio{$c_natio} = $natio;	# Also return value
	} # s_natio
    }


{   my %plaats;
    my @plaats;
    sub s_plaats ($)
    {
	my $c_plaats = shift;

	$cache{plaats}[0]++;

	defined $c_plaats or  return "";
	$c_plaats == -1   and return "";
	exists $plaats{$c_plaats} and return $plaats{$c_plaats};

	$cache{plaats}[1]++;

	@plaats == $cache{plaats}[2] and delete $plaats{shift @plaats};

	my $sth = $cache{plaats}[3];
	$sth->execute ($c_plaats);
	my ($plaats, $diac) = $sth->fetchrow_array;
	$diac and $plaats = diac ($plaats, $c_plaats, "v_003302", "plaats");
	push @plaats, $c_plaats;
	$plaats{$c_plaats} = $plaats;	# Also return value
	} # s_plaats
    }

{   my %gem_deel;
    my @gem_deel;
    sub s_gem_deel ($)
    {
	my $c_gem_deel = shift;

	$cache{gem_deel}[0]++;

	defined $c_gem_deel or  return "";
	$c_gem_deel == -1   and return "";
	exists $gem_deel{$c_gem_deel} and return $gem_deel{$c_gem_deel};

	$cache{gem_deel}[1]++;

	@gem_deel == $cache{gem_deel}[2] and delete $gem_deel{shift @gem_deel};

	my $sth = $cache{gem_deel}[3];
	$sth->execute ($c_gem_deel);
	my ($gem_deel, $diac) = $sth->fetchrow_array;
	$diac and $gem_deel = diac ($gem_deel, $c_gem_deel, "v_00bd02", "gem_deel");
	push @gem_deel, $c_gem_deel;
	$gem_deel{$c_gem_deel} = $gem_deel;	# Also return value
	} # s_gem_deel
    }

{   my %straat;
    my @straat;
    sub s_straat ($)
    {
	my $c_straat = shift;
	my @s = ("", "", "");	# straat, straat_l, straat_nen

	$cache{straat}[0]++;

	if (defined $c_straat && $c_straat != -1) {
	    if (exists $straat{$c_straat}) {
		@s = @{$straat{$c_straat}};
		}
	    else {
		$cache{straat}[1]++;

		@straat == $cache{straat}[2] and delete $straat{shift @straat};

		my $sth = $cache{straat}[3];
		$sth->execute ($c_straat);
		my ($straat, $diac, $straat_l, $diac_l,
		    $straat_nen, $diac_nen) = $sth->fetchrow_array;
		$diac     and $straat     = diac ($straat,     $c_straat, "v_00bc02", "straat");
		$diac_l   and $straat_l   = diac ($straat_l,   $c_straat, "v_00bc05", "straat");
		$diac_nen and $straat_nen = diac ($straat_nen, $c_straat, "v_00bc07", "straat");
		push @straat, $c_straat;
		@s = ($straat, $straat_l, $straat_nen);
		$straat{$c_straat} = [ @s ];
		}
	    }
	wantarray ? @s : $s[0];
	} # s_straat
    }

{   my %locatie;
    my @locatie;
    sub s_locatie ($)
    {
	my $c_locatie = shift;

	$cache{locatie}[0]++;

	defined $c_locatie or  return "";
	$c_locatie == -1   and return "";
	exists $locatie{$c_locatie} and return $locatie{$c_locatie};

	$cache{locatie}[1]++;

	@locatie == $cache{locatie}[2] and delete $locatie{shift @locatie};

	my $sth = $cache{locatie}[3];
	$sth->execute ($c_locatie);
	my ($locatie, $diac) = $sth->fetchrow_array;
	$diac and $locatie = diac ($locatie, $c_locatie, "v_00be01", "locatie");
	push @locatie, $c_locatie;
	$locatie{$c_locatie} = $locatie;	# Also return value
	} # s_locatie
    }

{   my %naam;
    my @naam;
    sub s_naam ($)
    {
	my $c_naam = shift;

	$cache{naam}[0]++;

	defined $c_naam or  return "";
	$c_naam == -1   and return "";
	exists $naam{$c_naam} and return $naam{$c_naam};

	$cache{naam}[1]++;

	@naam == $cache{naam}[2] and delete $naam{shift @naam};

	my $sth = $cache{naam}[3];
	$sth->execute ($c_naam);
	my ($naam, $diac) = $sth->fetchrow_array;
	$diac and $naam = diac ($naam, $c_naam, "v_00ai01", "naam");
	push @naam, $c_naam;
	$naam{$c_naam} = $naam;	# Also return value
	} # s_naam
    }

{   my %tp;
    my @tp;
    sub s_tp ($)
    {
	my $c_tp = shift;

	$cache{tp}[0]++;

	defined $c_tp or  return "";
	$c_tp == -1   and return "";
	exists $tp{$c_tp} and return $tp{$c_tp};

	$cache{tp}[1]++;

	@tp == $cache{tp}[2] and delete $tp{shift @tp};

	my $sth = $cache{tp}[3];
	$sth->execute ($c_tp);
	my ($tp) = $sth->fetchrow_array;
	push @tp, $c_tp;
	$tp{$c_tp} = $tp;	# Also return value
	} # s_tp
    }

{   my %voorv;
    my @voorv;
    sub s_voorv ($)
    {
	my $c_voorv = shift;

	$cache{voorv}[0]++;

	defined $c_voorv or  return "";
	$c_voorv == -1   and return "";
	exists $voorv{$c_voorv} and return $voorv{$c_voorv};

	$cache{voorv}[1]++;

	@voorv == $cache{voorv}[2] and delete $voorv{shift @voorv};

	my $sth = $cache{voorv}[3];
	$sth->execute ($c_voorv);
	my ($voorv) = $sth->fetchrow_array;
	push @voorv, $c_voorv;
	$voorv{$c_voorv} = $voorv;	# Also return value
	} # s_voorv
    }

{   my %voorn;
    my @voorn;
    sub s_voorn ($)
    {
	my $c_voorn = shift;

	$cache{voorn}[0]++;
	
	defined $c_voorn or  return "";
	$c_voorn == -1   and return "";
	exists $voorn{$c_voorn} and return $voorn{$c_voorn};

	$cache{voorn}[1]++;

	@voorn == $cache{voorn}[2] and delete $voorn{shift @voorn};

	my $sth = $cache{voorn}[3];
	$sth->execute ($c_voorn);
	my ($voorn, $diac) = $sth->fetchrow_array;
	$diac and $voorn = diac ($voorn, $c_voorn, "v_00ah01", "voorn");
	push @voorn, $c_voorn;
	$voorn{$c_voorn} = $voorn;	# Also return value
	} # s_voorn
    }

{   my %doc;
    my @doc;
    sub s_doc ($)
    {
	my $c_doc = shift;

	$cache{doc}[0]++;

	defined $c_doc or  return "";
	$c_doc == -1 and return "";
	$c_doc ==  0 and return ".";
	$c_doc ==  1 and return "PK";
	$c_doc ==  2 and return "PKA";
	exists $doc{$c_doc} and return $doc{$c_doc};

	$cache{doc}[1]++;

	@doc == $cache{doc}[2] and delete $doc{shift @doc};

	my $sth = $cache{doc}[3];
	$sth->execute ($c_doc);
	my ($doc, $diac) = $sth->fetchrow_array;
	$diac and $doc = diac ($doc, $c_doc, "v_00bk01", "doc");
	push @doc, $c_doc;
	$doc{$c_doc} = $doc;	# Also return value
	} # s_doc
    }

1;
