#!/pro/bin/perl -w

package PROCURA::Cache::LEP;

use strict;
use Exporter;
use Carp;
use vars   qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

$VERSION = "1.05";
@ISA     = ("Exporter", scalar caller);
@EXPORT  = qw(s_natio s_land s_plaats s_straat s_wijk s_locatie
	      s_aard s_cat s_soort s_richting
	      s_usr s_gezag
	      s_aant
	      $diac_ok
	      );

@EXPORT_OK   = (@EXPORT, qw(maxcache cachestats stopcache diac));
%EXPORT_TAGS = (
    util	=> [ qw(maxcache cachestats stopcache) ],
    "03"	=> [ qw(s_plaats s_land) ],
    );

use PROCURA::Diac;
use PROCURA::DBD;

DBDlogon (1);	# Presume client using DBD will logoff

### ###########################################################################

my %cache = (	# [ Requests, Misses, Cache Size, $sth ]
    natio	=> [ 0, 0,   50, prepar ("select natio",
					 "from   natio",
					 "where  c_natio    = ?")	],
    land	=> [ 0, 0,   75, prepar ("select land",
					 "from   land",
					 "where  c_land     = ?")	],
    plaats	=> [ 0, 0,  150, prepar ("select plaats",
					 "from   plaats",
					 "where  c_plaats   = ?")	],
    wijk	=> [ 0, 0,   10, prepar ("select wijk",
					 "from   wijk",
					 "where  c_wijk     = ?")	],
    straat	=> [ 0, 0,  150, prepar ("select straat, diac",
					 "from   straat",
					 "where  c_straat   = ?")	],
    locatie	=> [ 0, 0,   10, prepar ("select locatie",
					 "from   locatie",
					 "where  c_locatie  = ?")	],
    cat		=> [ 0, 0,   10, prepar ("select cat",
					 "from   cat",
					 "where  c_cat      = ?")	],
    aard	=> [ 0, 0,   10, prepar ("select aard",
					 "from   aard",
					 "where  c_aard     = ?")	],
    soort	=> [ 0, 0,   50, prepar ("select soort",
					 "from   soort",
					 "where  c_soort    = ?")	],
    richting	=> [ 0, 0,   50, prepar ("select richting",
					 "from   richting",
					 "where  c_richting = ?")	],
    gezag	=> [ 0, 0,   10, prepar ("select gezag",
					 "from   gezag",
					 "where  c_gezag    = ?")	],
    usr		=> [ 0, 0,   10, prepar ("select usr",
					 "from   usr",
					 "where  c_usr      = ?")	],
    diac	=> [ 0, 0,    0, prepar ("select diac",
					 "from   diac",
					 "where  veld       = ?",
					 "   and c_veld     = ?")	],
    aant	=> [ 0, 0,    0, prepar ("select v_aant, aant",
					 "from   aant",
					 "where  c_aant = ?")		],
    );

sub stopcache ()
{
    foreach my $r (keys %cache) {
	my $sth = $cache{$r}[3] or next;
	$sth->finish;
	$cache{$r}[3] = undef;
	}
    } # stopcache

END { stopcache (); }

sub maxcache ($;$)
{
    my ($t, $mc) = @_;
    exists $cache{$t} or carp "Table $t is not cached";
    defined $mc or return $cache{$t}[2];
    $cache{$t}[2] = $mc + 0;
    } # maxcache

sub cachestats ()
{
    print STDERR "\n";
    print STDERR "Table      Cache   Asked  Misses    Hits  HitRate\n";
    print STDERR "========== ===== ======= ======= ======= ========\n";
    foreach my $tab (sort { $cache{$b}[0] <=> $cache{$a}[0] } keys %cache) {
	$tab eq "diac" and next;
	my ($ask, $miss, $csz) = @{$cache{$tab}};
	my $rate = $ask ? 100 * ($ask - $miss) / $ask : 100.0;
	printf STDERR "%-10s %5d %7d %7d %7d %6.2f %%\n",
	    $tab, $csz, $ask, $miss, $ask - $miss, $rate;
	}
    } # cachestats

use vars qw($diac_ok);
$diac_ok = 1;
sub diac ($$$$)
{
    my ($f, $c, $v, $d) = @_;

    $diac_ok or return $f;
    my $sth = $cache{diac}[3];
    $sth->execute ($v, $c);
    my ($diac) = $sth->fetchrow_array;
    $diac and return diacgba ($f, $diac);
    warn ucfirst ($d), " $c DIAC empty!";
    $f;
    } # diac

{   my %natio;
    my @natio;
    sub s_natio ($)
    {
	my $c_natio = shift;

	$cache{natio}[0]++;

	defined $c_natio or  return "";
	$c_natio <=    0 and return "";
	exists $natio{$c_natio} and return $natio{$c_natio};

	$cache{natio}[1]++;

	@natio == $cache{natio}[2] and delete $natio{shift @natio};

	my $sth = $cache{natio}[3];
	$sth->execute ($c_natio);
	my ($natio) = $sth->fetchrow_array;
	push @natio, $c_natio;
	$natio{$c_natio} = $natio;	# Also return value
	} # s_natio
    }

{   my %land;
    my @land;
    sub s_land ($)
    {
	my $c_land = shift;

	$cache{land}[0]++;

	defined $c_land or  return "";
	$c_land <=    0 and return "";
	$c_land == 6030 and return "Nederland";
	exists $land{$c_land} and return $land{$c_land};

	$cache{land}[1]++;

	@land == $cache{land}[2] and delete $land{shift @land};

	my $sth = $cache{land}[3];
	$sth->execute ($c_land);
	my ($land) = $sth->fetchrow_array;
	push @land, $c_land;
	$land{$c_land} = $land;	# Also return value
	} # s_land
    }

{   my %plaats;
    my @plaats;
    sub s_plaats ($)
    {
	my $c_plaats = shift;

	$cache{plaats}[0]++;

	defined $c_plaats or  return "";
	$c_plaats <=    0 and return "";
	exists $plaats{$c_plaats} and return $plaats{$c_plaats};

	$cache{plaats}[1]++;

	@plaats == $cache{plaats}[2] and delete $plaats{shift @plaats};

	my $sth = $cache{plaats}[3];
	$sth->execute ($c_plaats);
	my ($plaats) = $sth->fetchrow_array;
	push @plaats, $c_plaats;
	$plaats{$c_plaats} = $plaats;	# Also return value
	} # s_plaats
    }

{   my %wijk;
    my @wijk;
    sub s_wijk ($)
    {
	my $c_wijk = shift;

	$cache{wijk}[0]++;

	defined $c_wijk or  return "";
	$c_wijk <=    0 and return "";
	exists $wijk{$c_wijk} and return $wijk{$c_wijk};

	$cache{wijk}[1]++;

	@wijk == $cache{wijk}[2] and delete $wijk{shift @wijk};

	my $sth = $cache{wijk}[3];
	$sth->execute ($c_wijk);
	my ($wijk) = $sth->fetchrow_array;
	push @wijk, $c_wijk;
	$wijk{$c_wijk} = $wijk;	# Also return value
	} # s_wijk
    }

{   my %straat;
    my @straat;
    sub s_straat ($)
    {
	my $c_straat = shift;

	$cache{straat}[0]++;

	defined $c_straat or  return "";
	$c_straat <=    0 and return "";
	exists $straat{$c_straat} and return $straat{$c_straat};

	$cache{straat}[1]++;

	@straat == $cache{straat}[2] and delete $straat{shift @straat};

	my $sth = $cache{straat}[3];
	$sth->execute ($c_straat);
	my ($straat, $diac) = $sth->fetchrow_array;
	s/\s+$// for $straat, $diac;
	$diac and $straat = diac ($straat, $c_straat, "straat", "straat");
	push @straat, $c_straat;
	$straat{$c_straat} = $straat;	# Also return value
	} # s_straat
    }

{   my %locatie;
    my @locatie;
    sub s_locatie ($)
    {
	my $c_locatie = shift;

	$cache{locatie}[0]++;

	defined $c_locatie or  return "";
	$c_locatie <=    0 and return "";
	exists $locatie{$c_locatie} and return $locatie{$c_locatie};

	$cache{locatie}[1]++;

	@locatie == $cache{locatie}[2] and delete $locatie{shift @locatie};

	my $sth = $cache{locatie}[3];
	$sth->execute ($c_locatie);
	my ($locatie) = $sth->fetchrow_array;
	push @locatie, $c_locatie;
	$locatie{$c_locatie} = $locatie;	# Also return value
	} # s_locatie
    }

{   my %cat;
    my @cat;
    sub s_cat ($)
    {
	my $c_cat = shift;

	$cache{cat}[0]++;

	defined $c_cat or  return "";
	$c_cat <=    0 and return "";
	exists $cat{$c_cat} and return $cat{$c_cat};

	$cache{cat}[1]++;

	@cat == $cache{cat}[2] and delete $cat{shift @cat};

	my $sth = $cache{cat}[3];
	$sth->execute ($c_cat);
	my ($cat) = $sth->fetchrow_array;
	push @cat, $c_cat;
	$cat{$c_cat} = $cat;	# Also return value
	} # s_cat
    }

{   my %soort;
    my @soort;
    sub s_soort ($)
    {
	my $c_soort = shift;

	$cache{soort}[0]++;

	defined $c_soort or  return "";
	$c_soort <=    0 and return "";
	exists $soort{$c_soort} and return $soort{$c_soort};

	$cache{soort}[1]++;

	@soort == $cache{soort}[2] and delete $soort{shift @soort};

	my $sth = $cache{soort}[3];
	$sth->execute ($c_soort);
	my ($soort) = $sth->fetchrow_array;
	push @soort, $c_soort;
	$soort{$c_soort} = $soort;	# Also return value
	} # s_soort
    }

{   my %richting;
    my @richting;
    sub s_richting ($)
    {
	my $c_richting = shift;

	$cache{richting}[0]++;

	defined $c_richting or  return "";
	$c_richting <=    0 and return "";
	exists $richting{$c_richting} and return $richting{$c_richting};

	$cache{richting}[1]++;

	@richting == $cache{richting}[2] and delete $richting{shift @richting};

	my $sth = $cache{richting}[3];
	$sth->execute ($c_richting);
	my ($richting) = $sth->fetchrow_array;
	push @richting, $c_richting;
	$richting{$c_richting} = $richting;	# Also return value
	} # s_richting
    }

{   my %aard;
    my @aard;
    sub s_aard ($)
    {
	my $c_aard = shift;

	$cache{aard}[0]++;

	defined $c_aard or  return "";
	$c_aard <=    0 and return "";
	exists $aard{$c_aard} and return $aard{$c_aard};

	$cache{aard}[1]++;

	@aard == $cache{aard}[2] and delete $aard{shift @aard};

	my $sth = $cache{aard}[3];
	$sth->execute ($c_aard);
	my ($aard) = $sth->fetchrow_array;
	push @aard, $c_aard;
	$aard{$c_aard} = $aard;	# Also return value
	} # s_aard
    }

{   my %usr;
    my @usr;
    sub s_usr ($)
    {
	my $c_usr = shift;

	$cache{usr}[0]++;

	defined $c_usr or  return "";
	$c_usr <=    0 and return "";
	exists $usr{$c_usr} and return $usr{$c_usr};

	$cache{usr}[1]++;

	@usr == $cache{usr}[2] and delete $usr{shift @usr};

	my $sth = $cache{usr}[3];
	$sth->execute ($c_usr);
	my ($usr) = $sth->fetchrow_array;
	push @usr, $c_usr;
	$usr{$c_usr} = $usr;	# Also return value
	} # s_usr
    }

{   my %gezag;
    my @gezag;
    sub s_gezag ($)
    {
	my $c_gezag = shift;

	$cache{gezag}[0]++;

	defined $c_gezag or  return "";
	$c_gezag <=    0 and return "";
	exists $gezag{$c_gezag} and return $gezag{$c_gezag};

	$cache{gezag}[1]++;

	@gezag == $cache{gezag}[2] and delete $gezag{shift @gezag};

	my $sth = $cache{gezag}[3];
	$sth->execute ($c_gezag);
	my ($gezag) = $sth->fetchrow_array;
	push @gezag, $c_gezag;
	$gezag{$c_gezag} = $gezag;	# Also return value
	} # s_gezag
    }

{   my ($v_aant, $aant);

    $cache{aant}[3]->bind_columns (\$v_aant, \$aant);

    sub s_aant ($)
    {
	my $c_aant = shift or return;
	my ($sth, @aant) = $cache{aant}[3];
	$sth->execute ($c_aant);
	while ($sth->fetch) {
	    $aant[$v_aant] = $aant;
	    }
	$aant[$_] //= "" for 0..$#aant;
	wantarray and return @aant;
	return join "\n" => @aant, "";
	} # s_aant
    }

1;
