package PROCURA::SQL;

$PROCURA::SQL::VERSION = 4.06;

=head1 NAME

SQL - supply perl-SQL interface

=head1 SYNOPSIS

    use PROCURA::SQL;

    my $cmd = sqlscript (<statement>);
    foreach (`$cmd | sort -u`) {
	:
	}

    @data = local_sql (<statement>);

    sql_debug (1);		# default 0
    sql_error ("/dev/tty");	# default /dev/null

=head1 DESCRIPTION

B<sqlscript ()> builds a SQL script in the correct syntax for the current
environment and return the command to execute the script. B<sqlscript ()>
uses I<$tmpfil> if defined,  otherwise I<$ENV{TMPFIL}> to compose the script
in.

B<local_sql ()> uses B<sqlscript ()> to build a SQL script for statement and
executes the returned command to return lines from SQL as a list. This
is equal to I<C<my $sql = sqlscript (E<lt>commandE<gt>); @data = `$sql`;>>.

B<sql_debug ()> sets the verbose level for PROCURA::SQL.S<
    0   Default. No messages.
    1   SQL statements are displayed on STDERR.
    2   DataBase variables are shown when creating SQL statements.>

B<sql_error ()>  defines where the error output of the  SQL command  will
go to. Default it will be trashed in I</dev/null>. B<sql_error ("/dev/tty")>
can be used to stop discarding the error output.

If there's no valid current environment,  define at least B<$ENV{DBPATH}>
(absolute path, I<no symbolic link!>),  B<$ENV{UNIFY}> and B<$ENV{PATH}> (to be
able to find the right SQL command).  In U2000 or  DataServer,  do not
forget to define B<$ENV{USCHEMA}> if applicable.


Bugs: poor adaption to different sql environments
   Er mogen GEEN newlines in sql queries voorkomen !!!

=cut

### ###########################################################################

use Exporter;
use vars qw(@ISA @EXPORT);

@ISA	 = qw(Exporter);
@EXPORT	 = qw(local_sql sqlscript sql_debug sql_error);

use IO::Handle;		# for autoflush () 
use PROCURA;		# for ftok ()
use strict;
use Carp; 

use vars qw($errlog $debug $lines0 $oldsig $sopt *tmpfil);
$debug   = 0;
$errlog  = "/dev/null";
$lines0  = 1;
$sopt    = "";
*tmpfil  = \$main::tmpfil;

# start SQL shell en verwerk opstart spam
sub BEGIN
{
    umask 0;

    $oldsig = $SIG{INT};
    $SIG{INT} = 'PROCURA::SQL::SigINT';
    } # BEGIN

sub unimport
{
    defined $oldsig and $SIG{INT} = $oldsig;
    } # unimport

sub END
{
    defined $oldsig and $SIG{INT} = $oldsig;
    } # END

sub sql_debug ($)
{
    $debug = shift;
    } # sql_debug

sub sql_error ($)
{
    $errlog = shift;
    } # sql_error

sub SigINT
{
    print "PROCURA::SQL::Interrupt !!!\n";
    delete $SIG{INT};
    } # SigINT

sub make_sql
{
    my $s = shift;
    my $sql;

    if (defined $ENV{USCHEMA}) {		# U2000, DataServer
	$sql = join "\n", ("lines 0;",
			   "SET TRANSACTION SCAN LEVEL 7;",
			   @{$s},
			   ";\n");
	$sopt = "-s $ENV{USCHEMA} -m 10";
	}
    elsif (defined $ENV{ORACLE_SID}) {		# Oracle
	$sql = join "\n", (@{$s},
			   ";\n");
	$sopt = "";	# SQL is a perl-wrapper 'round sqlplus
	}
    else {					# IDS
	$sql = join "\n", ("lines 0",
			   @{$s},
			   "/\n");
	$sopt = "";
	}
    $debug > 1 and print STDERR "[$ENV{UNIFY}] ".`db?`;
    $debug and print STDERR $sql;
    $sql;
    } # make_sql

sub local_sql
{
    my $tmp = $ENV{TMPDIR} || "/tmp";
    $tmp .= "/s.ql.$$";
    open  TMP, ">$tmp" or croak "Cannot create $tmp: $!";
    print TMP make_sql (\@_);
    close TMP;
    my $sql = "SQL $sopt $tmp 2>$errlog";
    $debug and print STDERR "Executing: $sql\n";
    open  TMP, "$sql |";
    my @sql = (<TMP>);
    close TMP;
    $debug < 8 and unlink $tmp;
    @sql;
    } # local_sql

sub sqlscript
{
    my $tmp = $tmpfil || $ENV{TMPFIL} ||
	croak "PROCURA::SQL:sqlscript() \$tmpfil undefined";
    open  TMP, ">$tmp" or croak "Cannot open $tmp: $!";
    print TMP make_sql (\@_);
    close TMP;
    my $sql = "SQL $sopt $tmp 2>$errlog";
    $debug and print STDERR "Executing: $sql\n";
    $sql;
    } # sqlscript

1;
