;# $header: terminfo.pl, v 1.0 94/07/12 13:45:00 hmb $
;#
;# usage:
;#	require "terminfo.pl";
;#	&tgetinf ("wy60");	# sets $TI{blink}, etc...

;# &tgetinf ([$TERM]);
sub tgetinf {
    local ($TERM) = @_;
    local ($TERMINFO, $_, $entry, $str);

    foreach $key (keys (%TI)) {
	delete $TI{$key};
	}

    $TERM	= $ENV{TERM}		unless $TERM;
    $TERMINFO	= $ENV{TERMINFO};
    $TERMINFO	= "/usr/lib/terminfo"	unless $TERMINFO =~ m/\S+/;

    # Be sure these are defined
    for my $attr (qw( blink bold dim invis pad prot rev rmacs rmso rmul
		      sgr sgr0 smacs smso smul )) {
	$TI{$attr} = "";
	}

    # Why bother the format if either dtic, untic or infocmp
    # knows it very well
    open (CMD, "untic $TERM|");
    while (<CMD>) {
	next if /\b$TERM\b/;
	chop;
	s/^\s+//;
	foreach $entry (split (m/, */, $_)) {
	    if ($entry =~ /^\w+$/) {
		$TI{$entry} = 1;
		}
	    elsif ($entry =~ /^(\w+)#(.*)/) {
		$TI{$1} = $2;
		}
	    elsif ($entry =~ /^(\w+)=(.*)/) {
		$str = $1;
		$_ = $2;
		s/\\E/\033/g;
		s/\\(\d\d\d)/pack ("C", oct ("0$1") & 0377)/eg;
		s/\\n/\n/g;
		s/\\r/\r/g;
		s/\\t/\t/g;
		s/\\b/\b/g;
		s/\\f/\f/g;
		s/\\s/ /g;
		s/\\\^/\377/g;
		s/\^\?/\177/g;
		s/\^(.)/pack ("c", ord ($1) & 31)/eg;
		s/\\(.)/$1/g;
		s/\377/^/g;
		$TI{$str} = $_;
		}
	    }
	}
    $TI{pad}   eq ""&& ($TI{pad}  = "\0");
    # Early version checked out existing values before overwriting
    if ($TI{sgr} =~ m/\?/) {
	$TI{smso}  = tparinf ($TI{sgr}, 1, 0, 0, 0, 0, 0, 0, 0, 0);
	$TI{smul}  = tparinf ($TI{sgr}, 0, 1, 0, 0, 0, 0, 0, 0, 0);
	$TI{rev}   = tparinf ($TI{sgr}, 0, 0, 1, 0, 0, 0, 0, 0, 0);
	$TI{blink} = tparinf ($TI{sgr}, 0, 0, 0, 1, 0, 0, 0, 0, 0);
	$TI{dim}   = tparinf ($TI{sgr}, 0, 0, 0, 0, 1, 0, 0, 0, 0);
	$TI{bold}  = tparinf ($TI{sgr}, 0, 0, 0, 0, 0, 1, 0, 0, 0);
	$TI{invis} = tparinf ($TI{sgr}, 0, 0, 0, 0, 0, 0, 1, 0, 0);
	$TI{prot}  = tparinf ($TI{sgr}, 0, 0, 0, 0, 0, 0, 0, 1, 0);
	$TI{smacs} = tparinf ($TI{sgr}, 0, 0, 0, 0, 0, 0, 0, 0, 1);
	}
    $TI{rmso}  eq ""&& ($TI{rmso} = $TI{sgr0});
    $TI{rmul}  eq ""&& ($TI{rmul} = $TI{sgr0});
    $TI{rmacs} eq ""&& ($TI{rmacs}= $TI{sgr0});
    close (CMD);
    } # tgetinf

;# &tputinf ($inf, $affcnt, FILEHANDLE);
sub tputinf {
    local ($str, $ac, $FH) = @_;
    local ($ms, $dcr);

    if ($str =~ /(^.*)\$<([\d.]+)(\*?)>(.*)$/) {
	$ms  = $2;
	$ms *= $ac if $3;
	$str = &tputinf ($1, $ac);
	$dcr = .1;		# depends on speed, the slower, the higher
	if ($dcr > .1) {
	    $ms  += $dcr / 2;
	    $str .= $TI{pad} x ($ms / $dcr);
	    }
	$str .= &tputinf ($4, $ac);
	}
    print $FH $str if $FH;
    $str;
    } # tputinf

;# $mv = &tparinf ($TI{cup}, $col, $row);
sub tparinf {
  local ($str)	= shift (@_);
  local ($res)	= "";
  local (@prm)	= @_;
  local (@stk)	= ();
  local ($ifc)	= 0;
  local (@ifb);
  local ($hdr, $pct);
  local (%var);

  foreach $pct ('a' .. 'z') {
      $var{$pct} = '';
      }
  while ($str =~ /^([^%]*)%(%|d|2d|3d|02d|03d|c|s|p[1-9]|P[a-z]|g[a-z]|'.'|{\d+}|l|[\+\-\*\/m]|[\&\|\^]|[\=\>\<]|[\!\~]|i|[\?te;])(.*)/) {
      $hdr  = $1;
      $pct  = $2;
      $str  = $3;
      if ($ifc <= 0 || $ifb[$ifc] > 0) {
	  $res .= $hdr;
	  if ($pct eq "%") {
	      $res .= "%";
	      }
	  elsif ($pct eq "d") {
	      $res .= sprintf ("%d",	pop (@stk));
	      }
	  elsif ($pct eq "2d") {
	      $res .= sprintf ("%2d",	pop (@stk));
	      }
	  elsif ($pct eq "3d") {
	      $res .= sprintf ("%3d",	pop (@stk));
	      }
	  elsif ($pct eq "02d") {
	      $res .= sprintf ("%02d",	pop (@stk));
	      }
	  elsif ($pct eq "03d") {
	      $res .= sprintf ("%03d",	pop (@stk));
	      }
	  elsif ($pct eq "c") {
	      $res .= sprintf ("%c",	pop (@stk));
	      }
	  elsif ($pct eq "s") {	# This calls for core-dumps
	      $res .= sprintf ("%s",	pop (@stk));
	      }
	  elsif ($pct =~ /^p([1-9])$/) {
	      push (@stk, $prm[$1 - 1]);
	      }
	  elsif ($pct =~ /^P([1-9])$/) {
	      $var{$1} = pop (@stk);
	      }
	  elsif ($pct =~ /^g([1-9])$/) {
	      push (@stk, $var{$1});
	      }
	  elsif ($pct =~ /^'(.)'$/) {
	      push (@stk, ord ($1));
	      }
	  elsif ($pct =~ /^\{(\d+)\}$/) {
	      push (@stk, int ($1));
	      }
	  elsif ($pct eq "l") {
	      push (@stk, length (pop (@stk)));
	      }
	  elsif ($pct eq "\+") {
	      push (@stk, pop (@stk) + pop (@stk));
	      }
	  elsif ($pct eq "-") {
	      push (@stk, pop (@stk) - pop (@stk));
	      }
	  elsif ($pct eq "\*") {
	      push (@stk, pop (@stk) * pop (@stk));
	      }
	  elsif ($pct eq "/") {
	      push (@stk, pop (@stk) / pop (@stk));
	      }
	  elsif ($pct eq "m") {
	      push (@stk, pop (@stk) % pop (@stk));
	      }
	  elsif ($pct eq "&") {
	      push (@stk, pop (@stk) & pop (@stk));
	      }
	  elsif ($pct eq "\|") {
	      push (@stk, pop (@stk) | pop (@stk));
	      }
	  elsif ($pct eq "\^") {
	      push (@stk, pop (@stk) ^ pop (@stk));
	      }
	  elsif ($pct eq "=") {
	      push (@stk, pop (@stk) == pop (@stk));
	      }
	  elsif ($pct eq "<") {
	      push (@stk, pop (@stk) << pop (@stk));
	      }
	  elsif ($pct eq ">") {
	      push (@stk, pop (@stk) >> pop (@stk));
	      }
	  elsif ($pct eq "!") {
	      push (@stk, !pop (@stk));
	      }
	  elsif ($pct eq "~") {
	      push (@stk, ~pop (@stk));
	      }
	  elsif ($pct eq "i") {
	      $prm[0] += 1;
	      $prm[1] += 1;
	      }
	  }
      if ($pct eq "\?") {
	  $ifb[++$ifc] = 2;
	  }
      elsif ($pct eq "t") {
	  $ifb[$ifc] = pop (@stk) > 0 ? 1 : 0 if $ifc > 0;
	  }
      elsif ($pct eq "e") {
	  $ifb[$ifc] = 1 - $ifb[$ifc] if $ifc > 0;
	  }
      elsif ($pct eq ";") {
	  $ifc-- if $ifc > 0;
	  }
      }
  $res . $str;
  } # tparinf

1;
